/*
 *
 * irdacapture.c - PSP IrDA Capture, v1.0 
 *
 * Written by Vanya A. Sergeev - <vsergeev@gmail.com>
 * 
 */
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <pspsircs.h>
#include <stdlib.h>

/* Define the module info section */
PSP_MODULE_INFO("IrDA Capture", 0, 1, 1);

/* Define the main thread's attribute value (optional) */
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

/* Convenient define for printf */
#define printf	pspDebugScreenPrintf

/* Prototype for irda capture function */
int irda_capture(int irdafd, int filefd);

/* Small delay loop */
void delay_loop(int num) {
	if (num <= 0) return;
	for (; num > 0; num--)
		sceDisplayWaitVblankStart();
}

/* Exit callback */
int exit_callback(void)
{
	sceKernelExitGame();
	return 0;
}

/* Callback thread */
void CallbackThread(void *arg)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

/* Quick IrDA Raw Capture */
int irda_capture(int irdafd, int filefd) {
	SceCtrlData pad;
	unsigned char data;
	int len;

	printf("Press the START button to stop.\n");
	while (1) {
		/* Read a byte */
		len = sceIoRead(irdafd, &data, 1);
		if (len != 0) {
			/* Print it out and put it in the file */
			printf("%c", data);
			len = sceIoWrite(filefd, &data, 1);
		}
		/* Check for the start button */
		sceCtrlReadBufferPositive(&pad, 1);
		if (pad.Buttons & PSP_CTRL_START)
			break;
	}

	printf("\n\nStopped capturing.\n");
	printf("Press any button to exit.\n");
	/* Wait till button pressed */
	pad.Buttons = 0;
	delay_loop(15);
	while (pad.Buttons == 0) {
		delay_loop(10);
		sceCtrlReadBufferPositive(&pad, 1);
	}
	return 0;
}


int main(int argc, char *argv[]) {
	SceCtrlData pad;
	char capturepath[] = "ms0:/irdacapture";
	int irdafd, filefd;
	
	SetupCallbacks();
	pspDebugScreenInit();

	sceCtrlSetSamplingCycle(0);
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_DIGITAL);
	
	printf("Simple PSP IrDA Capture v1.0 by Vanya Sergeev <vsergeev@gmail.com>\n\n");
	
	/* Open the irda file descriptor */
	irdafd = sceIoOpen("irda0:", PSP_O_RDWR, 0);
	if (irdafd < 0) {
		printf("ERROR: Could not open irda device!\n");
		printf("Press any button to exit...\n");
		while (pad.Buttons == 0) {
			delay_loop(2);
			sceCtrlReadBufferPositive(&pad, 1);
		}
		return -1;
	}

	printf("Opened IrDA device...\n");
	
	/* Open the file file descriptor */
	filefd = sceIoOpen(capturepath, PSP_O_WRONLY | PSP_O_CREAT | PSP_O_APPEND, 0777);
	if (filefd < 0) {
		printf("Error opening ms0:/irdacapture!\n");
		printf("Press any button to exit...\n");
		while (pad.Buttons == 0) {
			delay_loop(2);
			sceCtrlReadBufferPositive(&pad, 1);
		}
		return -1;
	}

	printf("Opened \"%s\" for writing...\n", capturepath);
	printf("Beginning capture...\n\n");

	/* Begin capture */
	irda_capture(irdafd, filefd);	

	/* Close irda and file */
	sceIoClose(irdafd);
	sceIoClose(filefd);
	
	/* Exit */
	sceKernelExitGame();
	
	return 0;
}

